(function ($) {
    /**
     * Finds any value in the Data Layer.
     *
     * @param key
     * @returns {*}
     */
    function wgmt_get_datalayer_value(key) {
        if (typeof dataLayer !== 'undefined') {
            const result = dataLayer.find(function (element) {
                return element[key];
            });

            return result[key];
        }
    }

    /**
     * Add new checkout step.
     *
     * @param step
     * @param option
     */
    function wgmt_add_checkout_step(step, option) {
        let data = {};

        data.step = step;

        if (typeof option !== 'undefined') {
            data.option = option;
        }

        dataLayer.push({"event": "checkoutStep", "checkoutStep": data});
    }

    $(document).ready(function () {
        /**
         * Checkout steps.
         */
        let current_view = wgmt_get_datalayer_value('current_view');

        if (typeof current_view !== 'undefined') {
            if (current_view === 'cart' && typeof eeMultiChanelVisitCartPageStep !== 'undefined') {
                wgmt_add_checkout_step(eeMultiChanelVisitCartPageStep);
            } else if (current_view === 'product' && typeof eeMultiChanelVisitProductPageStep !== 'undefined') {
                wgmt_add_checkout_step(eeMultiChanelVisitProductPageStep);
            } else if (current_view === 'checkout' && typeof eeMultiChanelVisitCheckoutStep !== 'undefined') {
                wgmt_add_checkout_step(eeMultiChanelVisitCheckoutStep);
            } else if (current_view === 'purchase' && typeof eeMultiChanelFinishOrderStep !== 'undefined') {
                wgmt_add_checkout_step(eeMultiChanelFinishOrderStep);
            }
        }

        /**
         * Capture add to cart event.
         */
        $(document.body).on('added_to_cart', function (e, fragments, cart_hash, this_button) {
            if (typeof dataLayer !== 'undefined') {
                let product_id = this_button[0]['dataset']['product_id'];
                let quantity = this_button[0]['dataset']['quantity'];

                let data = {
                    product_id: product_id,
                    quantity: quantity
                };

                $.ajax({
                    url: wgmt.ajaxUrl,
                    data: {action: 'wgmt_event_added_to_cart', data: data},
                    type: 'get',
                    dataType: 'json',
                    success: function (response) {
                        if (typeof eeMultiChanelAddToCartStep !== 'undefined') {
                            wgmt_add_checkout_step(eeMultiChanelAddToCartStep);
                        }

                        dataLayer.push({"event": "addToCart", "addToCart": response});
                    }
                });
            }
        });

        /**
         * Capture product click event.
         */
        $(document.body).on('click', 'a', function (event) {
            let link = this.href.replace(/&/g, '&amp;');

            if (typeof EeProductsClick !== 'undefined' && typeof EeProductsClick[link] !== 'undefined') {
                dataLayer.push({'event': 'productClick', 'productClicked': EeProductsClick[link]});
            }
        });
    });

    $(document).ajaxSuccess(function (event, xhr, settings, json) {
        /**
         * Capture remove from cart event.
         */
        if (settings.url.includes("remove_item")) {
            if (typeof dataLayer !== 'undefined') {
                let item_key = false;
                const url_params = new URLSearchParams(settings.url);
                for (let p of url_params) {
                    if (p[0].includes('remove_item')) {
                        item_key = p[1];
                    }
                }

                const cart_products = wgmt_get_datalayer_value('cart_products');
                cart_products.forEach(function (element, index, array) {
                    if (element['key'] === item_key) {
                        dataLayer.push({"event": "removeFromCart", "removeFromCart": element});
                    }
                });
            }
        }
    });
})(jQuery);
