(function ($) {
    /**
     * Toggle loader visibility.
     */
    function wgmt_toggle_spinner() {
        $("#wgmt_overlay").toggle();
    }

    function wgmt_store_notification(code, msg) {
        localStorage.setItem('notification_code', code);
        localStorage.setItem('notification_msg', msg);
    }

    function wgmt_clear_notification() {
        localStorage.removeItem('notification_code');
        localStorage.removeItem('notification_msg');
    }

    /**
     * Display WordPress native notice.
     *
     * @param type
     * @param msg
     */
    function display_notice(type, msg) {
        let code = 'notice-';

        switch (type) {
            case 'N':
                code += 'success';
                break;
            case 'W':
                code += 'warning';
                break;
            case 'S':
                code += 'info';
                break;
            case 'E':
            default:
                code += 'error';
                break;
        }

        let notice = $(".wgmt-hidden-notice");
        notice.addClass(code);

        let notice_p = notice.find("p").first();
        notice_p.html("<strong>Marketing Tools Pro for WooCommerce:</strong> " + msg);

        notice.slideDown("slow");
    }

    function wgmt_finish_action_with_notification(code, msg) {
        wgmt_store_notification(code, msg);
        wgmt_toggle_spinner();
        $('html, body').animate({scrollTop: 0}, 'fast');
        window.location.reload();
    }

    /**
     * Toggle section visibility.
     *
     * @param toggler
     */
    function wgmt_toggle_section(toggler) {
        let sectionName = toggler.data("sectionName");
        let sectionFields = $('form.google_marketing_tools-form').find('.section-' + sectionName);
        let togglerType = toggler.prop('nodeName');

        sectionFields.each(function () {
            if (togglerType === 'A') {
                $(this).slideToggle("slow");
            } else {
                if (toggler.is(':checked')) {
                    $(this).slideDown("slow");
                } else {
                    $(this).hide();
                }
            }
        });
    }

    /**
     * Switch tabs.
     *
     * @param tabs
     * @param tab
     */
    function wgmt_show_tab(tabs, tab) {
        if (typeof tab.data("showSave") === 'undefined') {
            $(".wgmt-save").hide();
        } else {
            $(".wgmt-save").show();
        }

        tabs.each(function () {
            $(this).removeClass("nav-tab-active");
        });

        let tab_containers = $(".tabs-content div.tab-content");
        tab_containers.each(function () {
            $(this).addClass("hidden");
        });

        tab.addClass("nav-tab-active");

        let tab_content = $(".tabs-content #" + tab.data("tab"));
        tab_content.removeClass("hidden");
    }

    /**
     * Go to a certain tab.
     *
     * @param tab
     */
    function wgmt_go_to_tab(tab) {
        let tabs = $("#google_marketing_tools_nav_tabs a");

        tabs.each(function () {
            let name = $(this).data("tab");
            if (name === "content_google_marketing_tools_tab_" + tab) {
                wgmt_show_tab(tabs, $(this));
            }
        });
    }

    /**
     * Save the taxonomy mapping.
     */
    function wgmt_save_mapping() {
        let taxonomyForm = $("form#google_marketing_tools_taxonomy");
        let data = taxonomyForm.serialize();

        $.ajax({
            url: wgmt.ajaxUrl,
            method: 'POST',
            data: {action: 'wgmt_save_taxonomy_mapping', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', 'Your taxonomy mapping was successfully saved!');
            },
            error: function (response) {
                wgmt_finish_action_with_notification('E', 'Error saving your mapping.');
            }
        });
    }

    /**
     * Load a taxonomy mapping.
     *
     * @param mapping_id
     */
    function wgmt_load_mapping(mapping_id) {
        let data = {
            mapping_id: mapping_id
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            method: 'GET',
            data: {action: 'wgmt_load_taxonomy_mapping', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                let card = $("#wgmt_card_taxonomies_new");
                card.slideDown("slow");
                $("#taxonomy_save_name").val(response.name);

                response.mapping.map((arr) => {
                    let select = $('select[name="taxonomy_category_name[' + arr.slug + ']"]');

                    let control = select[0].selectize;
                    control.addOption({id: arr.id, category: arr.category});
                    control.setValue(arr.id);
                });

                wgmt_toggle_spinner();
            },
            error: function (data) {
                wgmt_finish_action_with_notification('E', 'Error loading your mapping.');
            }
        });
    }

    /**
     * Delete a taxonomy map.
     *
     * @param mapping_id
     */
    function wgmt_delete_mapping(mapping_id) {
        let data = {
            mapping_id: mapping_id
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            method: 'POST',
            data: {action: 'wgmt_delete_taxonomy_mapping', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', 'Your taxonomy mapping was successfully deleted!');
            },
            error: function (response) {
                wgmt_finish_action_with_notification('E', 'Error deleting your mapping.');
            }
        });
    }

    function wgmt_load_feed_preset(merchant) {
        let data = {
            merchant: merchant
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            method: "get",
            data: {action: "wgmt_load_feed_preset", data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                let preset_table = $("#preset_table");
                $("#feeds_format").val(response.format);

                const attributes = Object.entries(response.attributes);
                wgmt_populate_attributes_select(attributes);

                const rows = Object.values(response.preset);
                wgmt_feeds_load_preset(rows);

                preset_table.slideDown("slow");

                wgmt_toggle_spinner();
            },
            error: function (response) {
                wgmt_finish_action_with_notification('E', 'Error loading merchant.');
            }
        });
    }

    /**
     * Populates the attributes select.
     *
     * @param attributes
     */
    function wgmt_populate_attributes_select(attributes) {
        let tbody = $("#preset_table").find("tbody").first();
        let origin = tbody.find("tr").first();
        let td = origin.find("td");

        let attribute_td = td.eq(0);
        let attribute_select = attribute_td.find("select").first();
        attribute_select.empty();

        attributes.map((arr) => {
            let optgroup = $("<optgroup label='" + arr[0] + "'>");

            const attrs = Object.entries(arr[1]);
            attrs.map((attr) => {
                let op = $("<option value='" + attr[0] + "'>" + attr[1] + "</option>");
                optgroup.append(op);
            });

            attribute_select.append(optgroup);
        });
    }

    /**
     * Load the merchant preset.
     *
     * @param rows
     */
    function wgmt_feeds_load_preset(rows) {
        let preset_table = $("#preset_table");
        let tbody = preset_table.find("tbody").first();
        let origin = tbody.find("tr").first();

        let index = 1;
        rows.map((arr) => {
            if (arr['mandatory']) {
                wgmt_feeds_render_row(index, arr);
                index++;
            }
        });
        origin.hide();
    }

    /**
     * Render a product attr row.
     *
     * @param index
     * @param custom
     * @param values
     */
    function wgmt_feeds_render_row(index, values = null, custom = false) {
        let preset_table = $("#preset_table");
        let tbody = preset_table.find("tbody").first();
        let origin = tbody.find("tr").first();

        let tr = origin.clone();
        tr.attr("data-attr-id", index);

        let cells = tr.find("td");
        let attribute_td = cells.eq(0);
        let value_td = cells.eq(1);
        let cdata_td = cells.eq(2);
        let if_td = cells.eq(3);
        let condition_td = cells.eq(4);

        // Indexes
        if (custom) {
            attribute_td.find("label").first().attr("for", "feeds_attr[" + index + "][custom]");
            attribute_td.find("select").first().remove();

            let input = $("<input/>").attr({
                type: "text",
                name: "feeds_attr[" + index + "][custom]"
            }).addClass("full-text");
            attribute_td.append(input);
        } else {
            attribute_td.find("label").first().attr("for", "feeds_attr[" + index + "][name]");
            attribute_td.find("select").first().attr("name", "feeds_attr[" + index + "][name]");
        }

        value_td.find("label").first().attr("for", "feeds_attr[" + index + "][value]");
        let value_select = value_td.find("select").first()
        value_select.attr("name", "feeds_attr[" + index + "][value]");
        value_select.on("change", function (event) {
            if (value_select.val() === 'string_static') {
                let input = $("<input/>").attr({
                    type: "text",
                    name: "feeds_attr[" + index + "][static]"
                }).addClass("full-text");
                value_td.append("<br>");
                value_td.append(input);
            }
            else {
                value_td.find("br").first().remove();
                value_td.find("input").first().remove();
            }
        });

        cdata_td.find("label").first().attr("for", "feeds_attr[" + index + "][cdata]");
        cdata_td.find("input").first().attr("name", "feeds_attr[" + index + "][cdata]");

        if_td.find("label").first().attr("for", "feeds_attr[" + index + "][if]");
        if_td.find("select").first().attr("name", "feeds_attr[" + index + "][if]");

        condition_td.find("label").first().attr("for", "feeds_attr[" + index + "][condition]");
        condition_td.find("input").first().attr("name", "feeds_attr[" + index + "][condition]");

        // Values
        if (null !== values) {
            attribute_td.find("select").first().val(values['attr']);
            value_td.find("select").first().val(values['taken']);
            cdata_td.find("input").first().prop("checked", values['cdata']);
        }

        // Delete button
        let remove_button = tr.find(".feeds-delete-attr");
        remove_button.on("click", function (event) {
            tr.remove();

            let rows = tbody.find("tr").length - 1;
            if (rows === 0) {
                preset_table.hide();

                let feedPresetButton = $("#feeds_preset");
                feedPresetButton.val("");
            }
        });

        tbody.append(tr);
        tr.show();
    }

    /**
     * Save and generate a feed.
     */
    function wgmt_save_feed() {
        let feedForm = $("form#google_marketing_tools_feeds");
        let data = feedForm.serialize();

        $.ajax({
            url: wgmt.ajaxUrl,
            method: "post",
            data: {action: 'wgmt_save_generate_feed', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', 'Your feed was successfully saved and generated!');
            },
            error: function (response) {
                wgmt_finish_action_with_notification('E', 'Error creating the feed, check your server permissions.');
            }
        });
    }

    /**
     * Load feed.
     *
     * @param feed_id
     */
    function wgmt_load_feed(feed_id) {
        let data = {
            feed_id: feed_id
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            method: "get",
            data: {action: 'wgmt_load_feed', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                /** Populate attributes field based on preset */
                const attributes = Object.entries(response.attributes);
                wgmt_populate_attributes_select(attributes);

                /** Load fields */
                let feed_config = Object.entries(response.feed.config);
                wgmt_load_feed_config(feed_config);

                /** Feed name */
                $("#feeds_save_name").val(response.feed.name);

                /** Display the feed */
                $(".wgmt-feeds-section-new").slideDown("slow");
                let preset_table = $("#preset_table");
                preset_table.slideDown("slow");

                wgmt_toggle_spinner();
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', JSON.parse(jqXHR.responseText).msg);
            }
        });
    }

    /**
     * Load feed config.
     *
     * @param feed_config
     */
    function wgmt_load_feed_config(feed_config) {
        let preset_table = $("#preset_table");
        let tbody = preset_table.find("tbody").first();
        let origin = tbody.find("tr").first();

        feed_config.map((arr) => {
            if (arr[0] === 'feeds_attr') {
                let presets = Object.values(arr[1]);
                let index = 1;

                presets.map((row) => {
                    let tr = origin.clone();
                    tr.attr("data-attr-id", index);

                    let cells = tr.find("td");
                    let attribute_td = cells.eq(0);
                    let value_td = cells.eq(1);
                    let cdata_td = cells.eq(2);
                    let if_td = cells.eq(3);
                    let condition_td = cells.eq(4);

                    /** Indexes */
                    if (row['custom']) {
                        attribute_td.find("label").first().attr("for", "feeds_attr[" + index + "][custom]");
                        attribute_td.find("select").first().remove();

                        let input = $("<input/>").attr({
                            type: "text",
                            name: "feeds_attr[" + index + "][custom]"
                        }).addClass("full-text");
                        attribute_td.append(input);
                    } else {
                        attribute_td.find("label").first().attr("for", "feeds_attr[" + index + "][name]");
                        attribute_td.find("select").first().attr("name", "feeds_attr[" + index + "][name]");
                    }

                    value_td.find("label").first().attr("for", "feeds_attr[" + index + "][value]");
                    value_td.find("select").first().attr("name", "feeds_attr[" + index + "][value]");

                    cdata_td.find("label").first().attr("for", "feeds_attr[" + index + "][cdata]");
                    cdata_td.find("input").first().attr("name", "feeds_attr[" + index + "][cdata]");

                    if_td.find("label").first().attr("for", "feeds_attr[" + index + "][if]");
                    if_td.find("select").first().attr("name", "feeds_attr[" + index + "][if]");

                    condition_td.find("label").first().attr("for", "feeds_attr[" + index + "][condition]");
                    condition_td.find("input").first().attr("name", "feeds_attr[" + index + "][condition]");

                    /** Values */
                    if (row['custom']) {
                        attribute_td.find("input").first().val(row['custom']);
                    } else {
                        attribute_td.find("select").first().val(row['name']);
                    }

                    value_td.find("select").first().val(row['value']);
                    cdata_td.find("input").first().prop("checked", row['cdata']);
                    if_td.find("select").first().val(row['if']);
                    condition_td.find("input").first().val(row['condition']);

                    tbody.append(tr);
                    tr.show();
                    index++;
                });

                origin.hide();
            }

            let field = $("*[name='" + arr[0] + "']");
            let input_type = field.attr("type");

            if (input_type === 'checkbox' && arr[1] === 'on') {
                field.click();
            } else if (input_type === 'radio') {
                field.each(function () {
                    if ($(this).val() === arr[1]) {
                        $(this).click();
                    }
                });
            } else {
                field.val(arr[1]);
            }

            let field_array = $("*[name='" + arr[0] + "[]']");
            let input_type_array = field_array.attr("type");

            if (input_type_array === 'checkbox') {
                field_array.each(function () {
                    if (arr[1].includes($(this).val())) {
                        $(this).click();
                    }
                });
            }
        });
    }

    /**
     * Delete a feed.
     *
     * @param feed_id
     */
    function wgmt_delete_feed(feed_id) {
        let data = {
            feed_id: feed_id
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            method: "post",
            data: {action: 'wgmt_delete_feed', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', response.msg);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', JSON.parse(jqXHR.responseText).msg);
            }
        });
    }

    /**
     * Generate Feed.
     *
     * @param feed_id
     */
    function wgmt_generate_feed(feed_id) {
        let data = {
            feed_id: feed_id
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            method: "post",
            data: {action: 'wgmt_generate_feed', data: data},
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', response.msg);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', JSON.parse(jqXHR.responseText).msg);
            }
        });
    }

    /**
     * Add a merchant attr row.
     */
    function wgmt_feeds_add_merchant_field() {
        let preset_table = $("#preset_table");
        let tbody = preset_table.find("tbody").first();
        let index = tbody.find("tr").length;

        wgmt_feeds_render_row(index);
    }

    function wgmt_feeds_add_custom_field() {
        let preset_table = $("#preset_table");
        let tbody = preset_table.find("tbody").first();
        let index = tbody.find("tr").length;

        wgmt_feeds_render_row(index, null, true);
    }

    /**
     * Saves configuration.
     */
    function wgmt_save_configuration() {
        let generalForm = $('form#google_marketing_tools_general');
        let workspaceForm = $('form#google_marketing_tools_workspace');
        let gdprForm = $('form#google_marketing_tools_gdpr');

        let data = generalForm.serialize() + '&' + workspaceForm.serialize() + '&' + gdprForm.serialize();

        $.ajax({
            method: 'post',
            url: wgmt.ajaxUrl,
            xhrFields: {withCredentials: false},
            data: {action: 'wgmt_save_configuration', data: data},
            dataType: 'json',
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', 'Your configuration was successfully saved!');
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', 'Error saving your configuration.');
            }
        });
    }

    /**
     * Restarts the tutoriasl.
     */
    function wgmt_restart_tutorial() {
        $.ajax({
            type: 'post',
            url: wgmt.ajaxUrl,
            xhrFields: {withCredentials: false},
            data: {action: 'wgmt_restart_tutorial'},
            dataType: 'json',
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_toggle_spinner();

                $('html, body').animate({scrollTop: 0}, 'fast');
                window.location.reload();
            }
        });
    }

    /**
     * Generate the workspace.
     */
    function wgmt_generate_workspace() {
        let workspaceForm = $('form#google_marketing_tools_workspace');
        let data = $(workspaceForm).find('input[type="text"], input[type="hidden"], input[type="checkbox"]:checked, select');

        $.ajax({
            type: 'post',
            url: wgmt.workspaceUrl,
            xhrFields: {withCredentials: false},
            data: data,
            dataType: 'json',
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification('N', response.notification_message);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', errorThrown + ': ' + jqXHR.responseJSON.notification_message);
            }
        });
    }

    /**
     * Open a ticket.
     */
    function wgmt_send_ticket() {
        let supportForm = $('form#google_marketing_tools_support');
        let data = $(supportForm).find('input[type="text"], input[type="hidden"], textarea');

        $.ajax({
            type: 'post',
            url: wgmt.ticketUrl,
            xhrFields: {withCredentials: false},
            data: data,
            dataType: 'json',
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                wgmt_finish_action_with_notification(response.notification_code, response.notification_message);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', errorThrown + ': ' + jqXHR.responseJSON.notification_message);
            }
        });
    }

    function wgmt_send_conversion(positive) {
        let negativeOrder = $("input#negative_conversion");
        let positiveOrder = $("input#positive_conversion");

        let order_id = positive === true ? positiveOrder.val() : negativeOrder.val();
        let gtm_id = $('input#container_id').val();

        let data = {
            status: positive,
            order_id: order_id,
            gtm_id: gtm_id
        };

        $.ajax({
            url: wgmt.ajaxUrl,
            data: {action: 'wgmt_send_conversion', data: data},
            type: 'post',
            dataType: 'json',
            beforeSend: function () {
                wgmt_toggle_spinner();
            },
            success: function (response) {
                $("head").append(response.script_head);
                $("body").append(response.script_body);

                wgmt_finish_action_with_notification('N', 'Your conversion was successfully send!');
            },
            error: function (jqXHR, textStatus, errorThrown) {
                wgmt_finish_action_with_notification('E', errorThrown + ': ' + jqXHR.responseJSON.msg);
            }
        });
    }

    /**
     * Document finish loading.
     */
    $(document).ready(function () {
        /** Tab to load */
        let tab = localStorage.getItem("tab");
        if (tab) {
            wgmt_go_to_tab(tab);
        }

        /** Manage sections */
        let sectionToggle = $('form.google_marketing_tools-form').find('.section-toggle');
        sectionToggle.each(function () {
            let togglerType = $(this).prop('nodeName');

            // initial visibility of the sections
            if (togglerType !== 'A') {
                wgmt_toggle_section($(this));
            }

            // change visibility on click
            $(this).on("click", function () {
                wgmt_toggle_section($(this));
            });
        });

        /** Manage tabs */
        let tabs = $("#google_marketing_tools_nav_tabs a");
        tabs.each(function () {
            $(this).on("click", function () {
                localStorage.setItem('tab', $(this).data("tabShort"));
                wgmt_show_tab(tabs, $(this));
            });
        });

        /** General select */
        $(".select2").selectize({
            create: false,
            persist: false,
        });

        /** Notifications */
        if (localStorage.getItem('notification_code')) {
            display_notice(localStorage.getItem('notification_code'), localStorage.getItem('notification_msg'));
            wgmt_clear_notification();
        }

        /**
         * General events
         ****************/
        /** Save configuration */
        $(".wgmt-save").on("click", function (event) {
            event.preventDefault();
            $(this).prop("disabled", true);

            wgmt_save_configuration();
        });

        /** Restart tutorial */
        $("#wgmt_restart_tutorial").on("click", function (event) {
            event.preventDefault();

            if (window.confirm("Restart the tutorial?")) {
                wgmt_restart_tutorial();
            }
        });

        /** Generate Workspace */
        $('.wgmt-workspace-btn').on("click", function (event) {
            event.preventDefault();
            $(this).prop("disabled", true);

            wgmt_generate_workspace();
        });

        /** Send ticket */
        $('#google_marketing_tools_send_ticket').on("click", function (event) {
            event.preventDefault();
            $(this).prop("disabled", true);

            wgmt_send_ticket();
        });

        /** Simulate conversions */
        let negativeConversionButton = $("#send_negative_conversion");
        negativeConversionButton.on("click", function (event) {
            wgmt_send_conversion(false);
        });

        let positiveConversionButton = $("#send_positive_conversion");
        positiveConversionButton.on("click", function (event) {
            wgmt_send_conversion(true);
        });

        /** Disable on click */
        $("#wgmt_license_form").on("submit", function (event) {
            $(".disable-on-click").prop("disabled", true);
        });

        /**
         * Taxonomy
         **********/
        /** Taxonomies select */
        $(".selectize-taxonomy").selectize({
            valueField: 'id',
            labelField: 'category',
            searchField: 'category',
            load: function (query, callback) {
                if (!query.length) return callback();

                let data = {
                    q: query,
                    page_limit: 10
                };

                $.ajax({
                    url: wgmt.ajaxUrl,
                    method: 'GET',
                    data: {action: 'wgmt_load_taxonomy_file', data: data},
                    beforeSend: function () {
                    },
                    error: function () {
                        callback();
                    },
                    success: function (response) {
                        callback(response);
                    }
                });
            }
        });

        /** New mapping */
        let newMappingButton = $("#taxonomy_new");
        newMappingButton.on("click", function (event) {
            $("#wgmt_card_taxonomies_new").slideDown("slow");
        });

        /** Save mapping */
        let saveMappingButton = $("#taxonomy_save_button");
        saveMappingButton.on("click", function (event) {
            wgmt_save_mapping();
        });

        /** Load mapping */
        let loadMappingButton = $("a.taxonomies-load-mapping");
        loadMappingButton.on("click", function (event) {
            event.preventDefault();
            wgmt_load_mapping($(this).data("mappingId"));
        });

        /** Delete mapping */
        let deleteMappingButton = $("button.taxonomy-delete-button");
        deleteMappingButton.on("click", function (event) {
            if (window.confirm("Do you really want to delete this mapping?")) {
                wgmt_delete_mapping($(this).data("mappingId"));
            }
        });

        /** Cancel */
        let cancelMappingButton = $("#taxonomy_cancel_button");
        cancelMappingButton.on("click", function (event) {
            $('html, body').animate({scrollTop: 0}, 'fast');
            $("#wgmt_card_taxonomies_new").slideUp("slow");
            $("#taxonomy_save_name").val("");
            $("#wgmt_card_taxonomies_new select").each(function () {
                let control = $(this)[0].selectize;
                control.clear();
            });
        });

        /**
         * Feeds
         *******/

        /** New feed */
        let newFeedButton = $("#feeds_new");
        newFeedButton.on("click", function (event) {
            $(".wgmt-feeds-section-new").slideDown("slow");
        });

        /** Cancel feeds */
        let feedCancelButton = $("#feeds_cancel_button");
        feedCancelButton.on("click", function (event) {
            window.location.reload();
        });

        /** Load preset */
        let feedPresetButton = $("#feeds_preset");
        feedPresetButton.on("change", function (event) {
            if ("" !== $(this).val()) {
                wgmt_load_feed_preset($(this).val());
            }
        });

        /** Save & Generate */
        let saveFeedButton = $("#feeds_save_button");
        saveFeedButton.on("click", function (event) {
            let valid = true;
            let preset = $("#feeds_preset");
            let name = $("#feeds_save_name");

            if (!preset.val()) {
                valid = false;
                window.confirm("Please, select a feed preset.");
            }

            if (!name.val()) {
                valid = false;
                window.confirm("Please, choose a feed name.");
            }

            if (valid) {
                wgmt_save_feed();
            }
        });

        /** Delete a feed */
        let deleteFeedButton = $(".feeds-delete-button");
        deleteFeedButton.on("click", function (event) {
            if (window.confirm("Do you really want to delete this feed?")) {
                wgmt_delete_feed($(this).data("feedId"));
            }
        });

        /** Generate feed */
        let generateFeedButton = $(".feeds-generate-button");
        generateFeedButton.on("click", function (event) {
            wgmt_generate_feed($(this).data("feedId"));
        });

        /** Add merchant field */
        let addMerchantFieldButton = $("#feeds_add_merchant_field");
        addMerchantFieldButton.on("click", function (event) {
            wgmt_feeds_add_merchant_field();
        });

        /** Add custom field */
        let addCustomFieldButton = $("#feeds_add_custom_field");
        addCustomFieldButton.on("click", function (event) {
            wgmt_feeds_add_custom_field();
        });

        /** Load feed */
        let loadFeedLink = $("a.feeds-load-feed");
        loadFeedLink.on("click", function (event) {
            event.preventDefault();

            wgmt_load_feed($(this).data("feedId"));
        });
    });
})(jQuery);
