<?php
/**
 * Plugin Name: Marketing Tools for WooCommerce
 * Version: 3.3.0
 * Plugin URI: https://devmanextensions.com/woocommerce-google-marketing-tools
 * Description: The perfect "All in One". Discover the most complete Google Tag Manager extension on WooCommerce.
 * Author: DevmanExtensions
 * Author URI: https://devmanextensions.com
 * Developer: David Nieves, Dariel Vicedo
 * Requires at least: 5.0
 * Tested up to: 5.6
 * Requires PHP: 7.0
 *
 * Text Domain: devman-marketing-tools
 * Domain Path: /i18n/languages/
 *
 * WC requires at least: 3.6.0
 * WC tested up to: 4.9.0
 *
 * @package WGMT
 */

defined( 'ABSPATH' ) || exit;

/**
 * Plugin constants.
 */
define( 'WGMT_PLUGIN_VERSION', '3.3.0' );
define( 'WGMT_PLUGIN_PLATFORM', 'woo' );
define( 'WGMT_PLUGIN_EXTENSION', 'gmt' );
define( 'WGMT_PLUGIN_NAME', 'devman-marketing-tools' );

if ( ! defined( 'WGMT_PLUGIN_FILE' ) ) {
	define( 'WGMT_PLUGIN_FILE', __FILE__ );
}

if ( ! defined( 'WGMT_PLUGIN_PATH' ) ) {
	define( 'WGMT_PLUGIN_PATH', plugin_dir_path( WGMT_PLUGIN_FILE ) );
}

if ( ! defined( 'WGMT_PLUGIN_URL' ) ) {
	define( 'WGMT_PLUGIN_URL', plugins_url() . '/' . WGMT_PLUGIN_NAME );
}

define( 'WGMT_PLUGIN_API', defined( 'DEVMAN_SERVER_TEST' ) ? DEVMAN_SERVER_TEST : 'https://api.devmanextensions.com/' );

/**
 * Actions on plugin activation / deactivation.
 */
function wgmt_activation() {
	if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
		wp_die( 'Sorry, but this plugin requires the WooCommerce Plugin to be installed and active. <br><a href="' . admin_url( 'plugins.php' ) . '">&laquo; Return to Plugins</a>' );
	}
}

register_activation_hook( WGMT_PLUGIN_FILE, 'wgmt_activation' );

/**
 * Feeds Cron.
 *
 * @param $store
 *
 * @throws Exception
 * @since 3.0.0
 */
function wgmt_feeds_cron_execute( $store ) {
	$store_values = WGMT()->wgmt_get_option( 'store_' . $store );

	$feeds = $store_values['feeds'];
	if ( ! empty( $feeds ) ) {
		$generated = false;

		foreach ( $feeds as $feed ) {
			$interval = array_key_exists( 'feeds_interval', $feed['config'] ) ? $feed['config']['feeds_interval'] : 'manually';
			if ( 'manually' !== $interval ) {
				switch ( $interval ) {
					case 'daily':
						$date_interval = new \DateInterval( "P1D" );
						break;
					case 'weekly':
						$date_interval = new \DateInterval( "P1W" );
						break;
					case 'monthly':
						$date_interval = new \DateInterval( "P1M" );
						break;
					default:
						$date_interval = new \DateInterval( "PT1H" );
						break;
				}

				/** @var DateTime $last_generation */
				$last_generation = $feed['generated'];

				$next_generation = clone $last_generation;
				$next_generation->add( $date_interval );
				$now = new DateTime();
				if ( $now >= $next_generation ) {
					$merchant_name = 'Merchant_' . $feed['config']['feeds_preset'];
					$merchant      = new $merchant_name( $feed );
					$merchant->wgmt_generate_feed();

					$generated = true;

					$store_values['feeds'][ $feed['id'] ]['generated'] = new DateTime();
				}
			}
		}

		if ( $generated ) {
			WGMT()->wgmt_set_option( 'store_' . $store, $store_values );
		}
	}
}

add_action( 'wgmt_feeds_cron_hook', 'wgmt_feeds_cron_execute' );

/**
 * Initialize the plugin.
 */
if ( ! class_exists( 'DevmanMarketingTools', false ) ) {
	include_once dirname( WGMT_PLUGIN_FILE ) . '/includes/class-wgmt-marketing-tools.php';
}

function WGMT() {
	return DevmanMarketingTools::instance();
}

// Global for backwards compatibility.
$GLOBALS['wgmt'] = WGMT();
