<?php
/**
 * WGMT Datalayer.
 *
 * @package WGMT
 * @since 2.0.0
 */

defined( 'ABSPATH' ) || exit;

final class WGMT_Datalayer {
	/**
	 * @var array
	 */
	protected $store_values;

	/**
	 * @var string
	 */
	protected $current_view;

	/**
	 * @var string
	 */
	protected $current_list;

	/**
	 * @var string
	 */
	protected $current_currency;

	/**
	 * @var int
	 */
	protected $user_id;

	/**
	 * @var string
	 */
	protected $search_query;

	/**
	 * @var array
	 */
	protected $cart_products;

	/**
	 * @var array
	 */
	protected $view_products;

	public static function wgmt_gtm_get_script( $container_id ) {
		$script = "\n<!-- BEGIN GTM Script by Google Marketing Tools Pro for WooCommerce -->\n";
		$script .= "<script type='text/javascript'>";

		$script .= "(function (w, d, s, l, i) {";
		$script .= "w[l] = w[l] || [];";
		$script .= "w[l].push({'gtm.start': new Date().getTime(), event: 'gtm.js'});";
		$script .= "var f = d.getElementsByTagName(s)[0], j = d.createElement(s), dl = l !== 'dataLayer' ? '&l=' + l : '';";
		$script .= "j.async = true;";
		$script .= "j.src = 'https://www.googletagmanager.com/gtm.js?id=' + i + dl;";
		$script .= "f.parentNode.insertBefore(j, f);";
		$script .= "})(window, document, 'script', 'dataLayer', '{$container_id}');";

		$script .= "</script>";
		$script .= "\n<!-- END GTM Script by Google Marketing Tools Pro for WooCommerce -->\n\n";

		return $script;
	}

	public static function wgmt_gtm_get_noscript( $container_id ) {
		$script = "<!-- BEGIN GTM by Google Marketing Tools Pro for WooCommerce -->";
		$script .= "<noscript>";
		$script .= "<iframe src='https://www.googletagmanager.com/ns.html?id={$container_id}' height='0' width='0' style='display: none; visibility: hidden;'></iframe>";
		$script .= "</noscript>";
		$script .= "<!-- END GTM by Google Marketing Tools Pro for WooCommerce -->";

		return $script;
	}

	public static function wgmt_send_backend_conversion( WC_Order $order, $status ) {
		$script = "<script type='text/javascript'>";
		$script .= "var dataLayer = [];";
		$script .= "dataLayer.push({'current_currency': '{$order->get_currency()}'});";
		$script .= "</script>";

		// Negative conversion?
		$order_total    = $status ? $order->get_total() : $order->get_total() * - 1;
		$order_tax      = $status ? $order->get_total_tax() : $order->get_total_tax() * - 1;
		$order_shipping = $status ? $order->calculate_shipping() : $order->calculate_shipping() * - 1;

		$order_data = json_encode( array(
			'id'          => $order->get_id(),
			'affiliation' => '',
			'revenue'     => number_format( (float) $order_total, '2', '.', '' ),
			'tax'         => number_format( (float) $order_tax, '2', '.', '' ),
			'shipping'    => number_format( (float) $order_shipping, '2', '.', '' ),
			'coupon'      => $order->get_coupon_codes(),
		) );

		$success_order_data = json_encode( array(
			'id'    => $order->get_id(),
			'total' => number_format( (float) $order_total, '2', '.', '' ),
		) );

		$products = array();
		/** @var \WC_Order_Item $order_item */
		foreach ( $order->get_items() as $order_item ) {
			$products[] = self::wgmt_format_cart_product( $order_item->get_product(), $order_item->get_quantity() );
		}
		$json_products = json_encode( $products );

		$script .= "<script type='text/javascript'>";

		// Enhanced E-commerce
		$script .= "dataLayer.push({'ecommerce': {";
		$script .= "'currencyCode': '{$order->get_currency()}',";
		$script .= "'purchase': {";
		$script .= "'actionField': {$order_data},";
		$script .= "'products': {$json_products}";
		$script .= "}";
		$script .= "}});";

		// GDPR
		$script .= "dataLayer.push({";
		$script .= "'gdpr_marketing_status': 'accepted', ";
		$script .= "'gdpr_statistics_status': 'accepted'";
		$script .= "});";

		// Success order
		$script .= "dataLayer.push({'event': 'orderSuccess', 'orderSuccess': {$success_order_data}});";

		// Event
		$script .= "dataLayer.push({'event': 'enhancedEcommercePurchase'});";

		$script .= "</script>";

		return $script;
	}

	public function wgmt_gtm_get_datalayer( $store_values ) {
		$this->store_values = $store_values;
		$this->datalayer_init_global_values();

		$script = "\n<!-- BEGIN GTM Datalayer by Devman Marketing Tools Pro for WooCommerce -->\n";
		$script .= "<script type='text/javascript'>";

		// variables
		$script .= "var dataLayer = [];";
		$script .= "var EeProductsClick = [];";

		if ( array_key_exists( 'multichannel_funnel_status', $store_values ) && $store_values['multichannel_funnel_status'] == 'on' ) {
			$script .= $this->datalayer_get_funnel_script();
		}

		// global values
		$script .= $this->datalayer_get_global_values();

		// Accept cookies if GDPR is not used
		if ( $store_values['gdpr_status'] != 'on' ) {
			$script .= 'dataLayer.push({"event":"GDPRMarketingAccepted", "gdpr_marketing_status":"accepted"});';
			$script .= 'dataLayer.push({"event":"GDPRStatisticsAccepted", "gdpr_statistics_status":"accepted"});';
		} else {
			if ( array_key_exists( 'wgmt_gdpr_marketing', $_COOKIE ) && array_key_exists( 'wgmt_gdpr_stats', $_COOKIE ) ) {
				if ( $_COOKIE['wgmt_gdpr_marketing'] == true ) {
					$script .= 'dataLayer.push({"event":"GDPRMarketingAccepted", "gdpr_marketing_status":"accepted"});';
				}

				if ( $_COOKIE['wgmt_gdpr_stats'] == true ) {
					$script .= 'dataLayer.push({"event":"GDPRStatisticsAccepted", "gdpr_statistics_status":"accepted"});';
				}
			}
		}

		// tags
		$script .= $this->datalayer_get_tags();

		$script .= "</script>";
		$script .= "\n<!-- END GTM Datalayer by Devman Marketing Tools Pro for WooCommerce -->\n";

		return $script;
	}

	public static function wgmt_format_cart_product( WC_Product $product, $quantity ) {
		/** @var WP_Term $category */
		if ( $product->get_type() == 'variation' ) {
			$category = wp_get_post_terms( $product->get_parent_id(), 'product_cat' )[0];
		} else {
			$category = wp_get_post_terms( $product->get_id(), 'product_cat' )[0];
		}

		return array(
			'id'       => $product->get_type() == 'variable' ? $product->get_parent_id() : $product->get_id(),
			'name'     => $product->get_name(),
			'price'    => number_format( (float) $product->get_price(), 2, '.', '' ),
			'brand'    => get_post_meta( $product->get_id(), 'brand', true ),
			'category' => $category->name,
			'variant'  => self::utils_get_product_variation( $product ),
			'quantity' => $quantity,
		);
	}

	/**
	 * Private
	 *********/

	private function datalayer_init_global_values() {
		$this->current_view     = $this->global_get_current_view();
		$this->current_list     = $this->global_get_current_list();
		$this->current_currency = get_woocommerce_currency();
		$this->user_id          = get_current_user_id() ?: 0;
		$this->search_query     = get_search_query();
		$this->cart_products    = $this->utils_get_formatted_cart_products();
		$this->view_products    = $this->utils_get_view_products();
	}

	private function datalayer_get_funnel_script() {
		$steps = array(
			'visit_product_page'  => $this->utils_get_funnel_step( 'visit_product_page' ),
			'add_to_cart'         => $this->utils_get_funnel_step( 'add_to_cart' ),
			'visit_cart_page'     => $this->utils_get_funnel_step( 'visit_cart_page' ),
			'visit_checkout_page' => $this->utils_get_funnel_step( 'visit_checkout_page' ),
			'finish_order'        => $this->utils_get_funnel_step( 'finish_order' ),
		);

		$script = "var eeMultiChanelVisitProductPageStep = {$steps['visit_product_page']};";
		$script .= "var eeMultiChanelAddToCartStep = {$steps['add_to_cart']};";
		$script .= "var eeMultiChanelVisitCartPageStep = {$steps['visit_cart_page']};";
		$script .= "var eeMultiChanelVisitCheckoutStep = {$steps['visit_checkout_page']};";
		$script .= "var eeMultiChanelFinishOrderStep = {$steps['finish_order']};";

		return $script;
	}

	/**
	 * Global values of the datalayer.
	 *
	 * @return string
	 */
	private function datalayer_get_global_values() {
		// common keys
		$script = 'dataLayer.push({"current_view":"' . $this->current_view . '"});';
		$script .= 'dataLayer.push({"current_list":"' . $this->current_list . '"});';
		$script .= 'dataLayer.push({"current_currency":"' . $this->current_currency . '"});';
		$script .= 'dataLayer.push({"userId":"' . $this->user_id . '"});';
		$script .= 'dataLayer.push({"string_searched":"' . $this->search_query . '"});';

		$json_cart_products = json_encode( $this->cart_products );
		$script             .= 'dataLayer.push({"cart_products":' . $json_cart_products . '});';

		// common events
		if ( $this->current_view == 'search' ) {
			$script .= 'dataLayer.push({"event":"isSearchView"});';
		}
		if ( $this->current_view == 'category' ) {
			$script .= 'dataLayer.push({"event":"isCategoryView"});';
		}

		return $script;
	}

	private function datalayer_get_tags() {
		$script = "";

		// populate products variable
		if ( ! empty( $this->view_products ) ) {
			/** @var \WC_Product $product */
			foreach ( $this->view_products as $product ) {
				if ( is_a( $product, 'WC_Product' ) ) {
					$json_product = json_encode( [
						'id'       => (string) $product->get_id(),
						'name'     => $product->get_name(),
						'price'    => number_format( (float) $product->get_price(), 2, '.', '' ),
						'brand'    => get_post_meta( $product->get_id(), 'brand', true ),
						'category' => $this->utils_get_category_list( $product->get_category_ids()[0] ),
						'list'     => $this->current_list,
					] );

					$script .= 'EeProductsClick["' . $product->get_permalink() . '"] = ' . $json_product . ';';
				}
			}
		}

		$tag_func = "datalayer_get_tags_view_{$this->current_view}";
		$script   .= $this->{$tag_func}();

		return $script;
	}

	private function datalayer_get_tags_view_homepage() {
		$script = "";

		// GA Impressions
		$impressions         = $this->utils_generate_ee_impressions( $this->view_products );
		$ga_impressions_data = json_encode( [
			'currencyCode' => $this->current_currency,
			'impressions'  => $impressions,
		] );
		$script              .= $this->get_script_ga_impressions( $ga_impressions_data );

		return $script;
	}

	private function datalayer_get_tags_view_shop() {
		return $this->datalayer_get_tags_view_homepage();
	}

	private function datalayer_get_tags_view_product() {
		/** @var \WC_Product $product */
		$product = wc_get_product();

		$script = "";

		if ( $this->current_view == 'product' ) {
			// GA Details
			$ga_details_data = json_encode( [
				'detail' => [
					'actionField' => [
						'list' => $this->current_list,
					],
					'products'    => [
						[
							[
								'id'       => $this->get_product_identifier( $product ),
								'name'     => $product->get_name(),
								'price'    => number_format( (float) $product->get_price(), 2, '.', '' ),
								'brand'    => get_post_meta( $product->get_id(), 'brand', true ),
								'category' => get_the_category_by_ID( $product->get_category_ids()[0] ),
								'quantity' => $product->get_stock_quantity(),
							],
						],
					],
				],
			] );
			$script          .= $this->get_script_ga_details( $ga_details_data );

			// GA Impressions
			$related_products = wc_get_related_products( $product->get_id() );
			$products         = [];
			foreach ( $related_products as $id ) {
				$products[] = wc_get_product( $id );
			}
			$impressions         = $this->utils_generate_ee_impressions( $products );
			$ga_impressions_data = json_encode( [
				'currencyCode' => $this->current_currency,
				'impressions'  => $impressions,
			] );
			$script              .= $this->get_script_ga_impressions( $ga_impressions_data );
		}

		return $script;
	}

	private function datalayer_get_tags_view_search() {
		$script = "";

		if ( $this->current_view == 'search' ) {
			$ids = [];
			foreach ( $this->view_products as $product ) {
				$ids[] = (string) $product->get_id();
			}

			// GA Impressions
			$impressions         = $this->utils_generate_ee_impressions( $this->view_products );
			$ga_impressions_data = json_encode( [
				'currencyCode' => $this->current_currency,
				'impressions'  => $impressions,
			] );
			$script              .= $this->get_script_ga_impressions( $ga_impressions_data );
		}

		return $script;
	}

	private function datalayer_get_tags_view_checkout() {
		return $this->datalayer_get_tags_view_cart();
	}

	private function datalayer_get_tags_view_cart() {
		return "";
	}

	private function datalayer_get_tags_view_purchase() {
		global $wp;

		$script = "";

		if ( $this->current_view == 'purchase' ) {
			$order_id = $wp->query_vars['order-received'];
			/** @var \WC_Order $order */
			$order = wc_get_order( $order_id );

			// prevents multiplied analytics on client multiple page refreshes.
			if ( $this->is_order_purchased( $order_id ) ) {
				return $script;
			}

			$ids = [];
			foreach ( $order->get_items() as $item ) {
				$ids[] = (string) $item->get_product()->{$this->store_values['google_ee_product_id_like']}();
			}

			// OrderSuccess
			$order_success_data = json_encode( [
				'id'    => (string) $order->get_id(),
				'total' => $order->get_total(),
			] );
			$script             .= $this->get_script_order_success( $order_success_data );

			// GA Purchase
			$products         = $this->utils_get_order_ee_products( $order );
			$ga_purchase_data = json_encode( [
				'currencyCode' => $this->current_currency,
				'purchase'     => [
					'actionField' => [
						'id'          => (string) $order->get_id(),
						'affiliation' => get_bloginfo( 'blogdescription' ),
						'revenue'     => $order->get_total( '' ),
						'tax'         => $order->get_total_tax(),
						'shipping'    => $order->calculate_shipping(),
						'coupon'      => $order->get_coupon_codes(),
					],
					'products'    => $products,
				],
			] );
			$script           .= $this->get_script_ga_purchase( $ga_purchase_data );
		}

		return $script;
	}

	private function datalayer_get_tags_view_category() {
		global $wp_query;
		global $current_user;

		$script = "";

		if ( $this->current_view == 'category' ) {
			$ids = [];
			/** @var \WC_Product $product */
			foreach ( $this->view_products as $product ) {
				$ids[] = (string) $product->{$this->store_values['google_ee_product_id_like']}();
			}

			// GA Impressions
			$impressions         = $this->utils_generate_ee_impressions( $this->view_products );
			$ga_impressions_data = json_encode( [
				'currencyCode' => $this->current_currency,
				'impressions'  => $impressions,
			] );
			$script              .= $this->get_script_ga_impressions( $ga_impressions_data );
		}

		return $script;
	}

	private function datalayer_get_tags_view_other() {
		return "";
	}

	private function get_script_ga_details( $data ) {
		return 'dataLayer.push({"event":"enhancedEcommerceProductDetails", "ecommerce":' . $data . '});';
	}

	private function get_script_ga_impressions( $data ) {
		return 'dataLayer.push({"event":"enhancedEcommerceProductImpressions", "ecommerce":' . $data . '});';
	}

	private function get_script_ga_purchase( $data ) {
		return 'dataLayer.push({"event":"enhancedEcommercePurchase", "ecommerce":' . $data . '});';
	}

	private function get_script_order_success( $data ) {
		return 'dataLayer.push({"event":"orderSuccess", "orderSuccess":' . $data . '});';
	}

	private function utils_generate_ee_impressions( $products ) {
		$impressions = [];

		/** @var \WC_Product $product */
		foreach ( $products as $index => $product ) {
			$impressions[] = [
				'id'       => $this->get_product_identifier( $product ),
				'name'     => $product->get_name(),
				'price'    => number_format( (float) $product->get_price(), 2, '.', '' ),
				'brand'    => get_post_meta( $product->get_id(), 'brand', true ),
				'category' => $this->utils_get_category_list( $product->get_category_ids()[0] ),
				'list'     => $this->current_list,
				'position' => $index + 1,
			];
		}

		return $impressions;
	}

	/**
	 * Get the current view name.
	 *
	 * @return string
	 */
	private function global_get_current_view() {
		if ( is_front_page() ) {
			return 'homepage';
		}

		if ( is_shop() ) {
			if ( is_search() ) {
				return 'search';
			}

			return 'shop';
		}

		if ( is_product_category() ) {
			return 'category';
		}

		if ( is_product() ) {
			return 'product';
		}

		if ( is_cart() ) {
			return 'cart';
		}

		if ( is_account_page() || is_edit_account_page() ) {
			return 'other';
		}

		if ( is_order_received_page() ) {
			return 'purchase';
		}

		if ( is_checkout() ) {
			return 'checkout';
		}

		return 'other';
	}

	private function global_get_current_list() {
		$list = $this->current_view;

		if ( $list == 'product' ) {
			$product = wc_get_product();
			$list    .= $product ? ': ' . $product->get_name() : '';

			return $list;
		}

		if ( $list == 'category' ) {
			global $wp_query;

			$category = $wp_query->get_queried_object();
			$list     = 'category: ' . $this->utils_get_formatted_category_list( $category->term_id );

			return $list;
		}

		if ( $list == 'search' ) {
			$q    = get_search_query();
			$list .= $q ? ': ' . $q : '';

			return $list;
		}

		return $list;
	}

	private function utils_get_formatted_category_list( $cat_id ) {
		$list  = "";
		$terms = $this->utils_get_category_list( $cat_id );

		foreach ( $terms as $term ) {
			$list .= $term . " > ";
		}

		return rtrim( $list, ' > ' );
	}

	private function utils_get_category_list( $cat_id ) {
		$terms = [];

		/** @var \WP_Term $term */
		$term = get_term( $cat_id );

		$terms[] = $term->name;
		while ( $term->parent != 0 ) {
			$term    = get_term( $term->parent );
			$terms[] = $term->name;
		}

		$terms = array_reverse( $terms );

		return $terms;
	}

	private function utils_get_formatted_cart_products() {
		global $woocommerce;

		$products = [];

		/** @var WC_Cart $cart */
		$cart = $woocommerce->cart;

		$cart_items = $cart->get_cart_contents();
		foreach ( $cart_items as $key => $value ) {
			/** @var \WC_Product $product */
			$product = $value['data'];

			/** @var WP_Term $category */
			if ( $product->get_type() == 'variation' ) {
				$category = wp_get_post_terms( $product->get_parent_id(), 'product_cat' )[0];
			} else {
				$category = wp_get_post_terms( $product->get_id(), 'product_cat' )[0];
			}

			$products[] = array(
				'id'       => $product->get_type() == 'variable' ? $product->get_parent_id() : $product->get_id(),
				'name'     => $product->get_name(),
				'price'    => number_format( (float) $product->get_price(), 2, '.', '' ),
				'brand'    => get_post_meta( $product->get_id(), 'brand', true ),
				'category' => $category->name,
				'variant'  => self::utils_get_product_variation( $product ),
				'quantity' => $value['quantity'],
				'key'      => $key,
			);
		}

		return $products;
	}

	private static function utils_get_product_variation( WC_Product $product ) {
		$variation = "";

		foreach ( $product->get_attributes() as $key => $attribute ) {
			if ( is_object( $attribute ) && get_class( $attribute ) == 'WC_Product_Attribute' ) {
				/** @var \WC_Product_Attribute $attribute */
				$name  = $attribute->get_name();
				$value = $product->get_attribute( $attribute->get_name() );

				if ( $value ) {
					$variation .= wc_attribute_label( $name ) . ': ' . $value . ', ';
				}
			} else {
				$value = $product->get_attribute( $key );

				if ( $value ) {
					$variation .= wc_attribute_label( $key ) . ': ' . $value . ', ';
				}
			}
		}

		return rtrim( $variation, ', ' );
	}

	private function utils_get_view_products() {
		global $wp_query;

		$products = [];

		$posts = $wp_query->posts;
		/** @var \WP_Post $post */
		foreach ( $posts as $post ) {
			if ( $post->post_type == 'product' ) {
				$products[] = wc_get_product( $post->ID );
			}
		}

		return $products;
	}

	private function utils_get_order_ee_products( \WC_Order $order ) {
		$products = [];
		$pos      = 1;

		/** @var \WC_Order_Item_Product $item */
		foreach ( $order->get_items() as $item ) {
			$category = ! empty( $item->get_product()->get_category_ids() ) ? get_the_category_by_ID( $item->get_product()->get_category_ids()[0] ) : '';

			$products[] = [
				'id'       => $this->get_product_identifier( $item->get_product() ),
				'name'     => $item->get_product()->get_name(),
				'price'    => $item->get_product()->get_price(),
				'brand'    => get_post_meta( $item->get_product()->get_id(), 'brand', true ),
				'category' => $category,
				'list'     => $this->current_list,
				'quantity' => $item->get_quantity(),
				'position' => $pos ++,
			];
		}

		return $products;
	}

	private function utils_get_funnel_step( $key ) {
		$steps = array(
			1 => $this->store_values['multichannel_step_1'],
			2 => $this->store_values['multichannel_step_2'],
			3 => $this->store_values['multichannel_step_3'],
			4 => $this->store_values['multichannel_step_4'],
			5 => $this->store_values['multichannel_step_5'],
		);

		return array_search( $key, $steps ) ?: 0;
	}

	private function is_order_purchased( $order_id ) {
		$session_orders = isset( $_SESSION['devman']['orders'] ) ? $_SESSION['devman']['orders'] : [];
		if ( in_array( $order_id, $session_orders ) ) {
			return true;
		}

		$_SESSION['devman']['orders'][] = $order_id;

		return false;
	}

	private function get_product_identifier( WC_Product $product ) {
		return (string) $product->{$this->store_values['google_ee_product_id_like']}();
	}
}
