<?php
/**
 * WGMT Storefront.
 *
 * @package WGMT
 * @since 2.0.0
 */

defined( 'ABSPATH' ) || exit;

class WGMT_Gdpr {
	/**
	 * @var array
	 */
	private $store_options;

	/**
	 * @var array
	 */
	private $config = array();

	public function __construct( $store_options ) {
		$this->store_options = $store_options;

		$this->load_config();
	}

	public function wgmt_gdpr_render() {
		$container_styles            = $this->wgmt_gdpr_get_container_styles();
		$h4_styles                   = $this->wgmt_gdpr_get_title_styles();
		$text_styles                 = $this->wgmt_gdpr_get_text_styles();
		$accept_styles               = $this->wgmt_gdpr_get_accept_button_styles();
		$configure_styles            = $this->wgmt_gdpr_get_configure_button_styles();
		$marketing_check_styles      = $this->wgmt_gdpr_get_marketing_checkbox_styles();
		$marketing_statistics_styles = $this->wgmt_gdpr_get_statistics_checkbox_styles();

		include_once WGMT_PLUGIN_PATH . '/includes/views/html-template-gdpr.php';

		if ( ! empty( $this->config['gdpr_badge_title'] ) ) {
			$gdpr_badge_styles     = $this->wgmt_gdpr_get_badge_styles();
			$gdpr_badge_img_styles = $this->wgmt_gdpr_get_badge_img_styles();

			include_once WGMT_PLUGIN_PATH . '/includes/views/html-template-gdpr-badge.php';
		}
	}

	private function load_config() {
		foreach ( $this->store_options as $key => $value ) {
			if ( substr( $key, 0, 4 ) === 'gdpr' ) {
				$this->config[ $key ] = $value;
			}
		}
	}

	/**
	 * Styles
	 ********/

	private function wgmt_gdpr_get_container_styles() {
		$alpha           = dechex( $this->config['gdpr_color_alpha'] );
		$primary         = $this->config['gdpr_color_back_primary'] . $alpha;
		$secondary       = $this->config['gdpr_color_back_secondary'] . $alpha;
		$background_type = $this->config['gdpr_choice_backtype'];

		$styles = "";

		if ( $background_type == 'solid' ) {
			$styles .= "background-color: {$primary} !important;";
		} elseif ( $background_type == 'horizontal' ) {
			$styles .= "background: linear-gradient(0.25turn, {$primary}, {$secondary}) !important;";
		} else {
			$styles .= "background: linear-gradient({$primary}, {$secondary}) !important;";
		}

		return $styles;
	}

	private function wgmt_gdpr_get_title_styles() {
		$color = $this->config['gdpr_color_titlebar'];

		$styles = "";

		$styles .= "color: {$color} !important;";

		return $styles;
	}

	private function wgmt_gdpr_get_text_styles() {
		$color = $this->config['gdpr_color_main'];

		$styles = "";

		$styles .= "color: {$color} !important;";

		return $styles;
	}

	private function wgmt_gdpr_get_accept_button_styles() {
		$background = $this->config['gdpr_color_accept_back'];
		$color      = $this->config['gdpr_color_accept_text'];

		$styles = "";

		$styles .= "background-color: {$background} !important;";
		$styles .= "color: {$color} !important;";

		return $styles;
	}

	private function wgmt_gdpr_get_configure_button_styles() {
		$background = $this->config['gdpr_color_configure_back'];
		$color      = $this->config['gdpr_color_configure_text'];

		$styles = "";

		$styles .= "background-color: {$background} !important;";
		$styles .= "color: {$color} !important;";

		return $styles;
	}

	private function wgmt_gdpr_get_marketing_checkbox_styles() {
		$color = $this->config['gdpr_color_marketing_check'];

		$styles = "";

		$styles .= "color: {$color} !important;";

		return $styles;
	}

	private function wgmt_gdpr_get_statistics_checkbox_styles() {
		$color = $this->config['gdpr_color_statistics_check'];

		$styles = "";

		$styles .= "color: {$color} !important;";

		return $styles;
	}

	private function wgmt_gdpr_get_badge_styles() {
		$background = $this->config['gdpr_color_back_primary'];

		$styles = "";

		switch ( $this->config['gdpr_badge_position'] ) {
			case '0':
				$styles .= "top: 0; left: 0;";
				$styles .= "border-top: 75px solid {$background} !important;";
				$styles .= "border-right: 75px solid transparent !important;";
				break;
			case '2':
				$styles .= "bottom: 0; right: 0;";
				$styles .= "border-bottom: 75px solid {$background} !important;";
				$styles .= "border-left: 75px solid transparent !important;";
				break;
			case '3':
				$styles .= "bottom: 0; left: 0;";
				$styles .= "border-bottom: 75px solid {$background} !important;";
				$styles .= "border-right: 75px solid transparent !important;";
				break;
			default:
				$styles .= "top: 0; right: 0;";
				$styles .= "border-top: 75px solid {$background} !important;";
				$styles .= "border-left: 75px solid transparent !important;";
				break;
		}

		return $styles;
	}

	private function wgmt_gdpr_get_badge_img_styles() {
		$styles = "";

		switch ( $this->config['gdpr_badge_position'] ) {
			case 0:
				$styles .= "top: 11px; left: 9px;";
				break;
			case 2:
				$styles .= "bottom: 11px; right: 9px;";
				break;
			case 3:
				$styles .= "bottom: 11px; left: 9px;";
				break;
			default:
				$styles .= "top: 11px; right: 9px;";
				break;
		}

		return $styles;
	}
}
