<?php
/**
 * Marketing Tools for WooCommerce.
 *
 * @package WGMT
 * @since   2.0.0
 */

defined( 'ABSPATH' ) || exit;

final class DevmanMarketingTools {
	/**
	 * The single instance of the class.
	 *
	 * @var DevmanMarketingTools
	 * @since 2.1.0
	 */
	protected static $_instance = null;

	/**
	 * The single instance of the WGMT_Backend class.
	 *
	 * @var WGMT_Backend
	 * @since 3.0.0
	 */
	protected static $_backend_instance = null;

	/**
	 * Options.
	 *
	 * @var array
	 */
	public static $wgmt_options = array();

	public function __construct() {
		self::wgmt_includes();

		self::$wgmt_options = null !== get_option( 'wgmt_options' ) ? get_option( 'wgmt_options' ) : array();

		// we love WooCommerce functions
		add_action( 'woocommerce_init', array( $this, 'wgmt_init' ) );
	}

	/**
	 * We go after WooCommerce.
	 *
	 * @since 2.0.0
	 */
	public function wgmt_init() {
		if ( is_admin() ) {
			self::$_backend_instance = WGMT_Backend::instance();
		} else {
			//  TODO: Integrate with a multistore plugin
			$working_store = 1;

			$store_options = null !== self::wgmt_get_option( 'store_' . $working_store ) ? self::wgmt_get_option( 'store_' . $working_store ) : array();
			$storefront    = new WGMT_Storefront( $store_options );
		}
	}

	protected static function wgmt_includes() {
		/**
		 * Core.
		 */
		if ( ! class_exists( 'WGMT_Backend', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/class-wgmt-backend.php';
		}

		if ( ! class_exists( 'WGMT_Helper', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/class-wgmt-helper.php';
		}

		if ( ! class_exists( 'WGMT_Storefront', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/class-wgmt-storefront.php';
		}

		if ( ! class_exists( 'WGMT_Datalayer', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/class-wgmt-datalayer.php';
		}

		/**
		 * Feeds.
		 */
		if ( ! class_exists( 'WGMT_Merchant', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/feeds/abstract-wgmt-merchant.php';
		}

		if ( ! class_exists( 'Merchant_Google_Shopping', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/feeds/class-wgmt-merchant-google-shopping.php';
		}

		/**
		 * GDPR
		 */
		if ( ! class_exists( 'WGMT_Gdpr', false ) ) {
			include_once WGMT_PLUGIN_PATH . 'includes/class-wgmt-gdpr.php';
		}
	}

	/**
	 * Main DevmanMarketingTools Instance.
	 *
	 * Ensures only one instance of DevmanMarketingTools is loaded or can be loaded.
	 *
	 * @return DevmanMarketingTools - Main instance.
	 * @see WGMT()
	 * @since 2.0.0
	 * @static
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Get a plugin option by key.
	 *
	 * @param $key
	 *
	 * @return mixed|null
	 *
	 * @since 2.0.0
	 */
	public function wgmt_get_option( $key ) {
		if ( array_key_exists( $key, (array) self::$wgmt_options ) ) {
			return self::$wgmt_options[ $key ];
		}

		return null;
	}

	/**
	 * Set a plugin option by key.
	 *
	 * @param $key
	 * @param $value
	 *
	 * @since 2.0.0
	 */
	public function wgmt_set_option( $key, $value ) {
		self::$wgmt_options[ $key ] = $value;

		update_option( 'wgmt_options', self::$wgmt_options );
	}

	/**
	 * Delete a plugin option by key.
	 *
	 * @param $key
	 *
	 * @since 2.0.0
	 */
	public function wgmt_delete_option( $key ) {
		if ( array_key_exists( $key, (array) self::$wgmt_options ) ) {
			unset( self::$wgmt_options[ $key ] );
			update_option( 'wgmt_options', self::$wgmt_options );
		}
	}
}
