<?php
/**
 * WGMT Storefront.
 *
 * @package WGMT
 * @since 2.0.0
 */

defined( 'ABSPATH' ) || exit;

class WGMT_Storefront {
	/**
	 * @var bool
	 */
	protected $plugin_status = false;

	/**
	 * @var bool
	 */
	protected $gdpr_status = false;

	/**
	 * The options for this store.
	 *
	 * @var array
	 */
	protected $store_options;

	/**
	 * The single instance of the WGMT_Datalayer class.
	 *
	 * @var WGMT_Datalayer
	 * @since 3.0.0
	 */
	protected $_datalayer_instance = null;

	public function __construct( array $store_options ) {
		unset( $store_options['feeds'] );
		unset( $store_options['taxonomies'] );
		$this->store_options = $store_options;

		if ( ( array_key_exists( 'tag_manager_status', $store_options ) && $store_options['tag_manager_status'] == 'on' ) && ( array_key_exists( 'container_id', $store_options ) && ! empty( $store_options['container_id'] ) ) ) {
			$this->plugin_status = true;
		}

		if ( array_key_exists( 'gdpr_status', $store_options ) && $store_options['gdpr_status'] == 'on' ) {
			$this->gdpr_status = true;
		}

		$this->init();
	}

	public function wgmt_load_gdpr_assets() {
		wp_enqueue_script( 'wgmt-cookies-script', WGMT_PLUGIN_URL . '/assets/js/js.cookie.min.js' );
		wp_enqueue_script( 'wgmt-gdpr-script', WGMT_PLUGIN_URL . '/assets/js/gdpr.js', array( 'jquery' ) );

		wp_localize_script( 'wgmt-gdpr-script', 'wgmt', array(
			'ajaxUrl' => admin_url( 'admin-ajax.php' ),
		) );
	}

	public function wgmt_gdpr_script() {
		echo "<script type='text/javascript'>var useWgmtGdpr = true;</script>";
	}

	public function wgmt_gdpr_render() {
		$gdpr = new WGMT_Gdpr( $this->store_options );

		$gdpr->wgmt_gdpr_render();
	}

	public function wgmt_load_storefront_assets() {
		wp_enqueue_script( 'wgmt-storefront-script', WGMT_PLUGIN_URL . '/assets/js/wgmt-events.js', array( 'jquery' ) );

		wp_localize_script( 'wgmt-storefront-script', 'wgmt', array(
			'ajaxUrl' => admin_url( 'admin-ajax.php' ),
		) );
	}

	/**
	 * Insert GTM Datalayer.
	 */
	public function wgmt_gtm_datalayer() {
		echo $this->_datalayer_instance->wgmt_gtm_get_datalayer( $this->store_options );
	}

	/**
	 * Insert GTM head script.
	 */
	public function wgmt_gtm_script() {
		echo WGMT_Datalayer::wgmt_gtm_get_script( $this->store_options['container_id'] );
	}

	/**
	 * Insert GTM body noscript.
	 */
	public function wgmt_gtm_noscript() {
		echo WGMT_Datalayer::wgmt_gtm_get_noscript( $this->store_options['container_id'] );
	}

	public function wgmt_event_add_to_cart() {
		if ( ! defined( 'DOING_AJAX' ) ) {
			$product_id = sanitize_text_field( $_POST['add-to-cart'] );
			$quantity   = sanitize_text_field( $_POST['quantity'] );

			$script = "<!-- BEGIN Add to cart event script by Devman Marketing Tools Pro for WooCommerce -->";
			$script .= "<script type='text/javascript'>";

			$script .= "var eeMultiChanelVisitProductPageStep = 0;";
			$script .= "if (eeMultiChanelAddToCartStep) { dataLayer.push({'event': 'checkoutStep', 'checkoutStep': eeMultiChanelAddToCartStep});}";

			// for product collections
			if ( is_array( $quantity ) ) {
				foreach ( $quantity as $product_id => $product_quantity ) {
					if ( $product_quantity > 0 ) {
						/** @var \WC_Product $product */
						$product = wc_get_product( $product_id );

						if ( is_a( $product, 'WC_Product' ) ) {
							$result = json_encode( WGMT_Datalayer::wgmt_format_cart_product( $product, $product_quantity ) );
							$script .= "dataLayer.push({'event': 'addToCart', 'addToCart': {$result}});";
						}
					}
				}
			} else {
				/** @var \WC_Product $product */
				$product = wc_get_product( $product_id );

				if ( is_a( $product, 'WC_Product' ) ) {
					$result = json_encode( WGMT_Datalayer::wgmt_format_cart_product( $product, $quantity ) );
					$script .= "dataLayer.push({'event': 'addToCart', 'addToCart': {$result}});";
				}
			}


			$script .= "</script>";
			$script .= "<!-- END Add to cart event script by Devman Marketing Tools Pro for WooCommerce -->";

			add_action( 'wp_head', function () use ( $script ) {
				echo $script;
			} );
		}
	}

	/**
	 * Private
	 *********/

	private function init() {
		if ( $this->plugin_status ) {
			$this->start_devman_session();

			$this->_datalayer_instance = new WGMT_Datalayer();

			// Assets
			add_action( 'wp_enqueue_scripts', array( $this, 'wgmt_load_storefront_assets' ) );

			// GTM Datalayer
			add_action( 'wp_head', array( $this, 'wgmt_gtm_datalayer' ) );

			// GTM main scripts
			add_action( 'wp_head', array( $this, 'wgmt_gtm_script' ) );
			add_action( 'wp_body_open', array( $this, 'wgmt_gtm_noscript' ) );

			// Events
			add_action( 'woocommerce_add_to_cart', array( $this, 'wgmt_event_add_to_cart' ) );
		}

		if ( $this->gdpr_status ) {
			// Variable
			add_action( 'wp_head', array( $this, 'wgmt_gdpr_script' ) );

			// Render the badge
			add_action( 'wp_footer', array( $this, 'wgmt_gdpr_render' ) );

			// Assets
			add_action( 'wp_enqueue_scripts', array( $this, 'wgmt_load_gdpr_assets' ) );
		}
	}

	/**
	 * Starts the devman session storage.
	 *
	 * @since 3.1.6
	 */
	private function start_devman_session() {
		session_start();
		if ( ! isset( $_SESSION['devman'] ) ) {
			$_SESSION['devman'] = [
				'orders' => [],
			];
		}
	}

	/**
	 * Debug the store options.
	 */
	private function debug_store_options() {
		print "<pre>";
		print_r( $this->store_options );
		print "</pre>";
	}
}
